@@ -1,6 +1,7 @@
 // Copyright 2020 ZomboDB, LLC <zombodb@gmail.com>. All rights reserved. Use of this source code is
 // governed by the MIT license that can be found in the LICENSE file.
 
+mod aggregate_tests;
 mod anyarray_tests;
 mod array_tests;
 mod bytea_tests;
@@ -13,6 +14,7 @@ mod fcinfo_tests;
 mod guc_tests;
 mod hooks_tests;
 mod inet_tests;
+mod internal_tests;
 mod json_tests;
 mod lifetime_tests;
 mod log_tests;
@@ -21,6 +23,7 @@ mod name_tests;
 mod numeric_tests;
 mod pg_extern_args_tests;
 mod pg_try_tests;
+mod pgbox_tests;
 mod postgres_type_tests;
 mod schema_tests;
 mod spi_tests;
